/* File PlayerEngine.h
 * Brief This file holds the declaration for class PlayerEngine
 *
 * Author
 *
 * Version
 *
 * Date
 *
 * This file gives the implementation of all the Media player operations
 *  such as play,pause etc using Phonon API.
 */

#ifndef __PLAYERINSTANCE_H__
#define __PLAYERINSTANCE_H__

       #include <string.h>
       #include <stdlib.h>
       #include <stdio.h>
       #include <sys/types.h>
       #include <sys/stat.h>
       #include <fcntl.h>
#include <unistd.h> 
       #include <sys/time.h>
       #include <sys/resource.h>

#include <gst/gst.h>
#include <glib.h>

//#define LOGGING
//#define X86


#define LOGGING
#ifdef LOGGING
#include <iostream>
using namespace std;
#define LOG2(str) cout << (str) << endl
#else
#define LOG2(str)
#endif

#define ETG_TRACE_USR2(str) LOG2(str)

#define ETG_TRACE_USR4 printf
#define ETG_TRACE_ERR printf
#define ETG_TRACE_FATAL printf
#define ETG_TRACE_ERR printf
#define GSTPLAYER_SUCCESS 0
#define GSTPLAYER_ERROR 1

// Calucalte number of hours, minutes and seconds from the milli seconds.
#define GET_HOURS(x)              (((x)/(60000*60)) % 60)
#define GET_MINUTES(x)            (((x)/(60000)) % 60)
#define GET_SECONDS(x)            (((x)/(1000)) % 60)


#define FORWARD (1)
#define REVERSE (-1)

/** Video layer order value if not set. */
#define VIDEO_LAYER_ORDER_NOT_SET (-1)
#define NUMBER_OF_CPU_LOAD_SAMPLES  (5)

//For AAC BT Playback
#define PATTERN_SIZE 5
#define ADTS_HEADER_SIZE 7
#define ADTS_FRAME_LENGTH(x) (((x[3] & 0x03) << 11) | (x[4] << 3) | ((x[5] & 0xe0) >> 5))
#define AAC_SAMPLES_PER_FRAME 1024
GMainLoop * g_main_loop;

/* This class provides interface over a dbus. Provides implementation of all the Media
player operations such as play,pause etc. */
class PlayerInstance
{


public:

    //Pipeline element
    GstElement *m_pPipeline;
	int m_goteos;
int inited;
	GstElement *m_ab;
	GstElement *m_file;
	GstElement *m_audio;
    //MetaData members
    //Stores Genre of the Song
    char *m_pGenre;

    //Stores Album of the Song
    char *m_pAlbum;

    //Stores Artist of the Song
    char *m_pArtist;

    //Stores Title of the Song
    char *m_pTitle;

    //Stores AudioCodec
    char *m_pAudioCodec;

    //Stores bitrate of the stream
    guint m_iBitrate;

    //Stores error Message
    char *m_pMessage;

    //Stores SourceName
    char *m_pSourceName;

    //Stores SourceType
    gint m_iSourceType;

    //Stores Duration
    gint64 m_iDuration;

    //Stores Duration
    gint64 m_iCurrentTime;

    //Stores whether event thread running or not
    gboolean m_bEventThreadRunning;

    //Stores Error Message

    //Stores Error Type
    int m_iErrorType;

    //For Layer synchronization

    //For Layer synchronization
    bool m_bStatus;

    //For remote playback
    //GMNGA-58138,GMNGA-58139
    bool m_isAlive;

    //to check the remote url type.(It can be http,rtsp url or a playlist)
    bool m_isPlaylist;
    //to store the buffering percentages
    gint m_iPercent;
    //to configure the initial playback based on the buffering percentage
    bool m_isInitialPlayback;
    //for storing the watchid of the messageloop.
    guint m_uiBusWatchId;

    //Find FR status for finished state call.
    bool m_bFindFRstatus;



    bool m_bSentBufferingMsg;

    //Creates Audio Elements
    int CreateAudioElements();



    //CallBack function for Pad added signal from URIDecodebin
    static void PadAddedCallback(GstElement *f_pDecodebin, GstPad *f_pPad, gpointer f_pUserData);

    //CallBack function for New Pad added signal from Decodebin
    static void NewPadAddedCallback(GstElement *f_pDecodebin, GstPad *f_pPad, gboolean f_bLast, gpointer f_pUserData);

    //CallBack function for Unknown added signal from Decodebin
    static void UnKnownTypeCallback (GstElement *f_pDecodebin, GstPad *f_pPad, GstCaps *f_pCaps, gpointer f_pUserData);

    //CallBack function for No More Pads added signal from Decodebin
    static void NoMorePadsCallback(GstElement * f_pDecodebin, gpointer f_pUserData);

    //CallBack function for No More Pads added signal from Uridecodebin
    static void NoMorePadsfromUridecodebin(GstElement * f_pDecodebin, gpointer f_pUserData);

    //CallBack function for rtsp source in case of pad added
    static void ConnectSourcePad(GstElement * f_pSource, GstPad *f_pPad, gpointer f_pUserData);

    //Reads data from pipe and provides it to appsrc
    static gboolean ProvideBTData(gpointer f_pUserData);

    static void FreeBTInputBuffer(void *f_pData);

    //Callback function for Gstreamer Messages
    static gboolean MessageLoop (GstBus *f_pBus, GstMessage *f_pMsg, gpointer f_pUserData);



    //Parses Error
    static void ParseError(GstMessage *f_pMsg, GError *f_pError, void *f_pUserData);

    static int ADTSSamplingRateFromIndex (unsigned int sr_idx);


    //parset metadata from Taglib
    void ParseTaglibData();

    //Sets Media Source for local or remote file for playback
    int SetMediaSource(char *f_ccMediaSrcPath);



    //Constructor
    PlayerInstance(char *path);

    //Destructor
    ~PlayerInstance();

    //Check if files exists or not.
    bool CheckIfFileExists(char *f_ccFileToplay);



    //Blocks until the thread has started or until time (l_ulTime) milliseconds have passed.
    //Returns true if the thread started successfully; otherwise returns
    //false (if the operation timed out or if an error occurred).
    bool waitForThreadStarted (unsigned long l_ulTime = ULONG_MAX);

    // This function parses the playlists i.e. URLs of form ".pls" and ".m3u"



    //This slot gets hit whenever there is a change in the playback state of MediaObject
    void StateChanged(int f_iState);

    //This slot gets hit whenever the MediaObject is about to finish playing.
    void AboutToFinish();

    //This slot gets hit whenever the MediaObject is finished playing.
    void Finished(bool f_bOnEOS = false);

    //If the MediaObject state is playing this slot gets hit after every half second.
    //and elapsed time is passed to this slot.

    void SetPipelineToStoppedState();


    //This function is used for play operation using Phonon
    void PlaySlot( char *f_ccFileToplay);

    void NewPlaySlot( char *f_ccFileToplay , int f_uiPlaypointPosition, int f_iPlaypointFormat );






private:


    //Stores Audio Sink Path
    char *m_ccAudioSinkDevicePath;

    //Stores Audio Source Path
    char *m_ccAudioSrcDevicePath;

    //Stores encoding type in case of pipe playback
    char *m_pEncodingType;

    //Constant ADTS header except size field
    static unsigned int m_uiADTSHeader[2];

    //Stores Seekable segment start
    gint64 m_iSeekableSegmentStart;

    //Stores Seekable segmentend
    gint64 m_iSeekableSegmentend;

    //Stores whether stream is seekable or not
    gboolean m_bSeekableStream;

    //Format of seeking
    GstFormat m_gstFormat;

    //Stores Playback rate
    gdouble m_dRate;

    //Stores Playback mode (FF or FR)
    gboolean m_bMode;

    //Stores Audio Sink Element
    GstElement *m_pAudioSink;

    //Stores Audio Convert Element
    GstElement *m_pAudioConvert;

    //Stores Audio Resample Element

    //Stores Audio Convert Capsfilter

    //Stores Volume Element
    GstElement *m_pVolume;

    //Stores video sink layer
    int m_ccVideoSinkLayer;

    //Stores Video Sink element
    GstElement *m_pVideoSink;

    //Stores Audio Queue Element
    GstElement *m_pAudioQueue;

    //Stores Video Queue Element
    GstElement *m_pVideoQueue;

    //Stores CapsFilter Element
    GstElement *m_pCapsFilter;

    //Stores whether audio stream exist or not
    gboolean m_bAudioStreamExist;

    //Stores whether video stream exist or not
    gboolean m_bVideoStreamExist;

    //Stores audiopad generated in new-pad-added callback
    GstPad *m_pAudioPad;

    //Stores videopad generated in new-pad-added callback
    GstPad *m_pVideoPad;

    //Stores whether metadata is changed or not
    gboolean m_bMetaDataChanged;

    // flag to avoid parsing of a second tag info in the same file
    gboolean m_bfirstTagInfo ;

    //Stores timer id created for sending elapsed time
    int m_iTimerID;

    //Stores timer id created for reading data from Pipe
    int m_pBtTimerID;

    //Stores timer id created for Forwading,Reversing
    int m_iTimerFRID;

    //Stores state of Gstreamer Pipeline
    int m_eState;

    //Status to store last
    int m_eStateBeforeTransition;

    //Stores the Display Width
    int m_iDisplayWidth;

    //Stores the Display Height
    int m_iDisplayHeight;
    //Stores whether player initialized or not
    gboolean m_bPlayerInitialized;

    //Stores whether seekable stream or not
    gboolean m_bSeeking;

    //Stores pipeline bus pointer
    GstBus *m_pBus;

    //Stores source element
    GstElement *m_pSource;

    //Stores Decodebin element
    GstElement *m_pDecodeBin;

    //Stores Pipe file descriptor
    gint m_iPipeFileDesc;

    //Creates Generic Pipeline
    int CreatePipeline();

    //Creates File Playback elements
    int CreateFilePlaybackElements(char *f_ccMediaSrcPath);

void ToStoppedState(GstElement *e);
    //Creates Alsa Playback elements
    int CreateAlsaPlaybackPipeline(char *f_pEncoding, int f_iSampleRate,
                                   int f_iChannels, int f_iSampleWidth,
                                   int f_iSampleDepth, bool f_bSigned,
                                   bool f_bEndianness);

    //Creates Pipe Playback elements
    int CreatePipePlaybackPipeline(char *f_pEncoding, int f_iBitRate);




    //Destroys Pipeline
    void DestroyPipeline();



    // Set to Pause
    bool SetStateToPause( char * f_ccFileTopause ,
                         int f_uiPausePointPosition, int f_iPausePointFormat);




    //Object Path
    char * m_sPath;

    //Fetchs the response of the server

    //Send network requests and receive replies



    // Variable used for iterating in a playlist in case first url is invalid
    int m_iPlaylstCount;



    //Current observer



    // Explicit stopping of Pipeline
    gboolean m_bExplicitStop;


    int m_iCpuLoadTimerCount;



};
#endif // __PLAYERINSTANCE_H__

